# Generated by asset_id_files.py

ABANDONED_WORKSITE_MREA = 0x54DAEF57
AMMO_STATION_MREA = 0x8697F62A
BROODING_GROUND_MREA = 0xDCEF49B9
CACHE_A_MREA = 0x0CAAEEC6
CACHE_B_MREA = 0x007551F9
CATACOMBS_MREA = 0xFB628DCB
CATACOMBS_ACCESS_MREA = 0xB2CAA0C9
CONTROLLER_ACCESS_MREA = 0x2AF65B49
CRYPT_MREA = 0xC8A84B98
CRYPT_TUNNEL_MREA = 0x4DC421AD
DARK_ARENA_TUNNEL_MREA = 0x390DFCB9
DARK_CONTROLLER_ACCESS_MREA = 0x11C4526B
DARK_FALLS_MREA = 0x2E259879
DARK_FORGOTTEN_BRIDGE_MREA = 0x0253782D
DARK_TORVUS_ARENA_MREA = 0xA49A572F
DARK_TORVUS_ENERGY_CONTROLLER_MREA = 0x5ED7CD19
DARK_TORVUS_TEMPLE_MREA = 0x3663B195
DARK_TORVUS_TEMPLE_ACCESS_MREA = 0xE597E4F9
DUNGEON_MREA = 0xBBE4B3AE
FORGOTTEN_BRIDGE_MREA = 0x09DECF21
FORTRESS_TRANSPORT_ACCESS_MREA = 0x63190A61
GATHERING_ACCESS_MREA = 0x530DFF81
GATHERING_HALL_MREA = 0x3501473A
GLOOM_VISTA_MREA = 0x954888C7
GREAT_BRIDGE_MREA = 0xE3D5A55E
GROVE_ACCESS_MREA = 0x46644C45
HYDROCHAMBER_STORAGE_MREA = 0x96A6E17E
HYDRODYNAMO_SHAFT_MREA = 0x308351E5
HYDRODYNAMO_STATION_MREA = 0xD5B5B066
MAIN_ENERGY_CONTROLLER_MREA = 0xA6803305
MAIN_HYDROCHAMBER_MREA = 0xCEBAC0BD
MEDITATION_VISTA_MREA = 0x5DFA32DB
PATH_OF_ROOTS_MREA = 0x7448931C
PLAZA_ACCESS_MREA = 0xC1C28706
POISONED_BOG_MREA = 0x14E2C3A6
POLLUTED_MIRE_MREA = 0x839DC4F6
PORTAL_CHAMBER_DARK_MREA = 0xAE1E1339
PORTAL_CHAMBER_LIGHT_MREA = 0xF3EE585F
PUTRID_ALCOVE_MREA = 0xCBC72D79
RUINED_ALCOVE_MREA = 0xD37C2414
SACRIFICIAL_CHAMBER_MREA = 0x629790F4
SACRIFICIAL_CHAMBER_TUNNEL_MREA = 0x65E571CD
SAVE_STATION_1_MREA = 0x9A777442
SAVE_STATION_2_MREA = 0x13254EC6
SAVE_STATION_A_MREA = 0xDB6E4075
SAVE_STATION_B_MREA = 0xCB165BD8
TEMPLE_ACCESS_MREA = 0x0740B4D3
TEMPLE_TRANSPORT_ACCESS_MREA = 0x27D2C898
TORVUS_ENERGY_CONTROLLER_MREA = 0x133BF5B8
TORVUS_GROVE_MREA = 0xE838FA18
TORVUS_LAGOON_MREA = 0x7E0BE699
TORVUS_MAP_STATION_MREA = 0x282B587F
TORVUS_PLAZA_MREA = 0x75F158FF
TORVUS_TEMPLE_MREA = 0xE640CE96
TRAINING_ACCESS_MREA = 0x77AF9FD1
TRAINING_CHAMBER_MREA = 0x4BB5AE60
TRANSIT_TUNNEL_EAST_MREA = 0x66F6300E
TRANSIT_TUNNEL_SOUTH_MREA = 0xB7259AA6
TRANSIT_TUNNEL_WEST_MREA = 0xE150FB4D
TRANSPORT_TO_AGON_WASTES_MREA = 0xCF659F4E
TRANSPORT_TO_SANCTUARY_FORTRESS_MREA = 0xBF0EE428
TRANSPORT_TO_TEMPLE_GROUNDS_MREA = 0x6F6515F2
UNDERGROUND_TRANSPORT_MREA = 0x9258DC82
UNDERGROUND_TUNNEL_MREA = 0x80E67F90
UNDERTEMPLE_MREA = 0x85A3801F
UNDERTEMPLE_ACCESS_MREA = 0x29B990F8
UNDERTEMPLE_SHAFT_MREA = 0xB2D3E26C
UNDERTRANSIT_ONE_MREA = 0x5CD4C2F9
UNDERTRANSIT_TWO_MREA = 0x4BFCA639
VENOMOUS_POND_MREA = 0x858EECD3

NAME_TO_ID_MREA = {
    "Abandoned Worksite": 0x54DAEF57,
    "Ammo Station": 0x8697F62A,
    "Brooding Ground": 0xDCEF49B9,
    "Cache A": 0x0CAAEEC6,
    "Cache B": 0x007551F9,
    "Catacombs": 0xFB628DCB,
    "Catacombs Access": 0xB2CAA0C9,
    "Controller Access": 0x2AF65B49,
    "Crypt": 0xC8A84B98,
    "Crypt Tunnel": 0x4DC421AD,
    "Dark Arena Tunnel": 0x390DFCB9,
    "Dark Controller Access": 0x11C4526B,
    "Dark Falls": 0x2E259879,
    "Dark Forgotten Bridge": 0x0253782D,
    "Dark Torvus Arena": 0xA49A572F,
    "Dark Torvus Energy Controller": 0x5ED7CD19,
    "Dark Torvus Temple": 0x3663B195,
    "Dark Torvus Temple Access": 0xE597E4F9,
    "Dungeon": 0xBBE4B3AE,
    "Forgotten Bridge": 0x09DECF21,
    "Fortress Transport Access": 0x63190A61,
    "Gathering Access": 0x530DFF81,
    "Gathering Hall": 0x3501473A,
    "Gloom Vista": 0x954888C7,
    "Great Bridge": 0xE3D5A55E,
    "Grove Access": 0x46644C45,
    "Hydrochamber Storage": 0x96A6E17E,
    "Hydrodynamo Shaft": 0x308351E5,
    "Hydrodynamo Station": 0xD5B5B066,
    "Main Energy Controller": 0xA6803305,
    "Main Hydrochamber": 0xCEBAC0BD,
    "Meditation Vista": 0x5DFA32DB,
    "Path of Roots": 0x7448931C,
    "Plaza Access": 0xC1C28706,
    "Poisoned Bog": 0x14E2C3A6,
    "Polluted Mire": 0x839DC4F6,
    "Portal Chamber (Dark)": 0xAE1E1339,
    "Portal Chamber (Light)": 0xF3EE585F,
    "Putrid Alcove": 0xCBC72D79,
    "Ruined Alcove": 0xD37C2414,
    "Sacrificial Chamber": 0x629790F4,
    "Sacrificial Chamber Tunnel": 0x65E571CD,
    "Save Station 1": 0x9A777442,
    "Save Station 2": 0x13254EC6,
    "Save Station A": 0xDB6E4075,
    "Save Station B": 0xCB165BD8,
    "Temple Access": 0x0740B4D3,
    "Temple Transport Access": 0x27D2C898,
    "Torvus Energy Controller": 0x133BF5B8,
    "Torvus Grove": 0xE838FA18,
    "Torvus Lagoon": 0x7E0BE699,
    "Torvus Map Station": 0x282B587F,
    "Torvus Plaza": 0x75F158FF,
    "Torvus Temple": 0xE640CE96,
    "Training Access": 0x77AF9FD1,
    "Training Chamber": 0x4BB5AE60,
    "Transit Tunnel East": 0x66F6300E,
    "Transit Tunnel South": 0xB7259AA6,
    "Transit Tunnel West": 0xE150FB4D,
    "Transport to Agon Wastes": 0xCF659F4E,
    "Transport to Sanctuary Fortress": 0xBF0EE428,
    "Transport to Temple Grounds": 0x6F6515F2,
    "Underground Transport": 0x9258DC82,
    "Underground Tunnel": 0x80E67F90,
    "Undertemple": 0x85A3801F,
    "Undertemple Access": 0x29B990F8,
    "Undertemple Shaft": 0xB2D3E26C,
    "Undertransit One": 0x5CD4C2F9,
    "Undertransit Two": 0x4BFCA639,
    "Venomous Pond": 0x858EECD3,
}
# Generated by asset_id_files.py

ABANDONED_WORKSITE_MAPA = 0x7D0F146A
AMMO_STATION_MAPA = 0xAF420D17
BROODING_GROUND_MAPA = 0xF53AB284
CACHE_A_MAPA = 0x257F15FB
CACHE_B_MAPA = 0x29A0AAC4
CATACOMBS_MAPA = 0xD2B776F6
CATACOMBS_ACCESS_MAPA = 0x9B1F5BF4
CONTROLLER_ACCESS_MAPA = 0x0323A074
CRYPT_MAPA = 0xE17DB0A5
CRYPT_TUNNEL_MAPA = 0x6411DA90
DARK_ARENA_TUNNEL_MAPA = 0x10D80784
DARK_CONTROLLER_ACCESS_MAPA = 0x3811A956
DARK_FALLS_MAPA = 0x07F06344
DARK_FORGOTTEN_BRIDGE_MAPA = 0x2B868310
DARK_TORVUS_ARENA_MAPA = 0x8D4FAC12
DARK_TORVUS_ENERGY_CONTROLLER_MAPA = 0x77023624
DARK_TORVUS_TEMPLE_MAPA = 0x1FB64AA8
DARK_TORVUS_TEMPLE_ACCESS_MAPA = 0xCC421FC4
DUNGEON_MAPA = 0x92314893
FORGOTTEN_BRIDGE_MAPA = 0x200B341C
FORTRESS_TRANSPORT_ACCESS_MAPA = 0x4ACCF15C
GATHERING_ACCESS_MAPA = 0x7AD804BC
GATHERING_HALL_MAPA = 0x1CD4BC07
GLOOM_VISTA_MAPA = 0xBC9D73FA
GREAT_BRIDGE_MAPA = 0xCA005E63
GROVE_ACCESS_MAPA = 0x6FB1B778
HYDROCHAMBER_STORAGE_MAPA = 0xBF731A43
HYDRODYNAMO_SHAFT_MAPA = 0x1956AAD8
HYDRODYNAMO_STATION_MAPA = 0xFC604B5B
MAIN_ENERGY_CONTROLLER_MAPA = 0x8F55C838
MAIN_HYDROCHAMBER_MAPA = 0xE76F3B80
MEDITATION_VISTA_MAPA = 0x742FC9E6
PATH_OF_ROOTS_MAPA = 0x5D9D6821
PLAZA_ACCESS_MAPA = 0xE8177C3B
POISONED_BOG_MAPA = 0x3D37389B
POLLUTED_MIRE_MAPA = 0xAA483FCB
PORTAL_CHAMBER_DARK_MAPA = 0x87CBE804
PORTAL_CHAMBER_LIGHT_MAPA = 0xDA3BA362
PUTRID_ALCOVE_MAPA = 0xE212D644
RUINED_ALCOVE_MAPA = 0xFAA9DF29
SACRIFICIAL_CHAMBER_MAPA = 0x4B426BC9
SACRIFICIAL_CHAMBER_TUNNEL_MAPA = 0x4C308AF0
SAVE_STATION_1_MAPA = 0xB3A28F7F
SAVE_STATION_2_MAPA = 0x3AF0B5FB
SAVE_STATION_A_MAPA = 0xF2BBBB48
SAVE_STATION_B_MAPA = 0xE2C3A0E5
TEMPLE_ACCESS_MAPA = 0x2E954FEE
TEMPLE_TRANSPORT_ACCESS_MAPA = 0x0E0733A5
TORVUS_ENERGY_CONTROLLER_MAPA = 0x3AEE0E85
TORVUS_GROVE_MAPA = 0xC1ED0125
TORVUS_LAGOON_MAPA = 0x57DE1DA4
TORVUS_MAP_STATION_MAPA = 0x01FEA342
TORVUS_PLAZA_MAPA = 0x5C24A3C2
TORVUS_TEMPLE_MAPA = 0xCF9535AB
TRAINING_ACCESS_MAPA = 0x5E7A64EC
TRAINING_CHAMBER_MAPA = 0x6260555D
TRANSIT_TUNNEL_EAST_MAPA = 0x4F23CB33
TRANSIT_TUNNEL_SOUTH_MAPA = 0x9EF0619B
TRANSIT_TUNNEL_WEST_MAPA = 0xC8850070
TRANSPORT_TO_AGON_WASTES_MAPA = 0xE6B06473
TRANSPORT_TO_SANCTUARY_FORTRESS_MAPA = 0x96DB1F15
TRANSPORT_TO_TEMPLE_GROUNDS_MAPA = 0x46B0EECF
UNDERGROUND_TRANSPORT_MAPA = 0xBB8D27BF
UNDERGROUND_TUNNEL_MAPA = 0xA93384AD
UNDERTEMPLE_MAPA = 0xAC767B22
UNDERTEMPLE_ACCESS_MAPA = 0x006C6BC5
UNDERTEMPLE_SHAFT_MAPA = 0x9B061951
UNDERTRANSIT_ONE_MAPA = 0x750139C4
UNDERTRANSIT_TWO_MAPA = 0x62295D04
VENOMOUS_POND_MAPA = 0xAC5B17EE

NAME_TO_ID_MAPA = {
    "Abandoned Worksite": 0x7D0F146A,
    "Ammo Station": 0xAF420D17,
    "Brooding Ground": 0xF53AB284,
    "Cache A": 0x257F15FB,
    "Cache B": 0x29A0AAC4,
    "Catacombs": 0xD2B776F6,
    "Catacombs Access": 0x9B1F5BF4,
    "Controller Access": 0x0323A074,
    "Crypt": 0xE17DB0A5,
    "Crypt Tunnel": 0x6411DA90,
    "Dark Arena Tunnel": 0x10D80784,
    "Dark Controller Access": 0x3811A956,
    "Dark Falls": 0x07F06344,
    "Dark Forgotten Bridge": 0x2B868310,
    "Dark Torvus Arena": 0x8D4FAC12,
    "Dark Torvus Energy Controller": 0x77023624,
    "Dark Torvus Temple": 0x1FB64AA8,
    "Dark Torvus Temple Access": 0xCC421FC4,
    "Dungeon": 0x92314893,
    "Forgotten Bridge": 0x200B341C,
    "Fortress Transport Access": 0x4ACCF15C,
    "Gathering Access": 0x7AD804BC,
    "Gathering Hall": 0x1CD4BC07,
    "Gloom Vista": 0xBC9D73FA,
    "Great Bridge": 0xCA005E63,
    "Grove Access": 0x6FB1B778,
    "Hydrochamber Storage": 0xBF731A43,
    "Hydrodynamo Shaft": 0x1956AAD8,
    "Hydrodynamo Station": 0xFC604B5B,
    "Main Energy Controller": 0x8F55C838,
    "Main Hydrochamber": 0xE76F3B80,
    "Meditation Vista": 0x742FC9E6,
    "Path of Roots": 0x5D9D6821,
    "Plaza Access": 0xE8177C3B,
    "Poisoned Bog": 0x3D37389B,
    "Polluted Mire": 0xAA483FCB,
    "Portal Chamber (Dark)": 0x87CBE804,
    "Portal Chamber (Light)": 0xDA3BA362,
    "Putrid Alcove": 0xE212D644,
    "Ruined Alcove": 0xFAA9DF29,
    "Sacrificial Chamber": 0x4B426BC9,
    "Sacrificial Chamber Tunnel": 0x4C308AF0,
    "Save Station 1": 0xB3A28F7F,
    "Save Station 2": 0x3AF0B5FB,
    "Save Station A": 0xF2BBBB48,
    "Save Station B": 0xE2C3A0E5,
    "Temple Access": 0x2E954FEE,
    "Temple Transport Access": 0x0E0733A5,
    "Torvus Energy Controller": 0x3AEE0E85,
    "Torvus Grove": 0xC1ED0125,
    "Torvus Lagoon": 0x57DE1DA4,
    "Torvus Map Station": 0x01FEA342,
    "Torvus Plaza": 0x5C24A3C2,
    "Torvus Temple": 0xCF9535AB,
    "Training Access": 0x5E7A64EC,
    "Training Chamber": 0x6260555D,
    "Transit Tunnel East": 0x4F23CB33,
    "Transit Tunnel South": 0x9EF0619B,
    "Transit Tunnel West": 0xC8850070,
    "Transport to Agon Wastes": 0xE6B06473,
    "Transport to Sanctuary Fortress": 0x96DB1F15,
    "Transport to Temple Grounds": 0x46B0EECF,
    "Underground Transport": 0xBB8D27BF,
    "Underground Tunnel": 0xA93384AD,
    "Undertemple": 0xAC767B22,
    "Undertemple Access": 0x006C6BC5,
    "Undertemple Shaft": 0x9B061951,
    "Undertransit One": 0x750139C4,
    "Undertransit Two": 0x62295D04,
    "Venomous Pond": 0xAC5B17EE,
}
# Generated by asset_id_files.py

ABANDONED_WORKSITE_INTERNAL_ID = 0x8F9D66C0
AMMO_STATION_INTERNAL_ID = 0xA0E73B00
BROODING_GROUND_INTERNAL_ID = 0x3C156DDF
CACHE_A_INTERNAL_ID = 0x97B812A2
CACHE_B_INTERNAL_ID = 0xF04E9059
CATACOMBS_INTERNAL_ID = 0xAD9D5D9F
CATACOMBS_ACCESS_INTERNAL_ID = 0xC4F8E78E
CONTROLLER_ACCESS_INTERNAL_ID = 0x94182918
CRYPT_INTERNAL_ID = 0x4280A031
CRYPT_TUNNEL_INTERNAL_ID = 0xA29D33C8
DARK_ARENA_TUNNEL_INTERNAL_ID = 0xA1ECA232
DARK_CONTROLLER_ACCESS_INTERNAL_ID = 0x65B4E891
DARK_FALLS_INTERNAL_ID = 0x9172B3EB
DARK_FORGOTTEN_BRIDGE_INTERNAL_ID = 0xFF42758B
DARK_TORVUS_ARENA_INTERNAL_ID = 0xF5F50ABC
DARK_TORVUS_ENERGY_CONTROLLER_INTERNAL_ID = 0x8E2AF917
DARK_TORVUS_TEMPLE_INTERNAL_ID = 0x915E47CE
DARK_TORVUS_TEMPLE_ACCESS_INTERNAL_ID = 0x6DB75FB4
DUNGEON_INTERNAL_ID = 0xFE8931FC
FORGOTTEN_BRIDGE_INTERNAL_ID = 0x2406DD54
FORTRESS_TRANSPORT_ACCESS_INTERNAL_ID = 0x518D3511
GATHERING_ACCESS_INTERNAL_ID = 0x3AD40963
GATHERING_HALL_INTERNAL_ID = 0xF9F219B0
GLOOM_VISTA_INTERNAL_ID = 0x279818B4
GREAT_BRIDGE_INTERNAL_ID = 0x069D4087
GROVE_ACCESS_INTERNAL_ID = 0x734255B0
HYDROCHAMBER_STORAGE_INTERNAL_ID = 0x0112E539
HYDRODYNAMO_SHAFT_INTERNAL_ID = 0x3CE4A4FB
HYDRODYNAMO_STATION_INTERNAL_ID = 0x2A621228
MAIN_ENERGY_CONTROLLER_INTERNAL_ID = 0xDE01532E
MAIN_HYDROCHAMBER_INTERNAL_ID = 0x50039A19
MEDITATION_VISTA_INTERNAL_ID = 0xA701265D
PATH_OF_ROOTS_INTERNAL_ID = 0xC746378D
PLAZA_ACCESS_INTERNAL_ID = 0xE14A2729
POISONED_BOG_INTERNAL_ID = 0xA7DA4FD1
POLLUTED_MIRE_INTERNAL_ID = 0x8D13B482
PORTAL_CHAMBER_DARK_INTERNAL_ID = 0x9F510909
PORTAL_CHAMBER_LIGHT_INTERNAL_ID = 0x554E4514
PUTRID_ALCOVE_INTERNAL_ID = 0x0C8B7C06
RUINED_ALCOVE_INTERNAL_ID = 0x1D951459
SACRIFICIAL_CHAMBER_INTERNAL_ID = 0xC1AE6ECF
SACRIFICIAL_CHAMBER_TUNNEL_INTERNAL_ID = 0x7FF3F046
SAVE_STATION_1_INTERNAL_ID = 0x59E57214
SAVE_STATION_2_INTERNAL_ID = 0x0D580FBE
SAVE_STATION_A_INTERNAL_ID = 0x964465B0
SAVE_STATION_B_INTERNAL_ID = 0xA74BE553
TEMPLE_ACCESS_INTERNAL_ID = 0xE2F4F72A
TEMPLE_TRANSPORT_ACCESS_INTERNAL_ID = 0xAA2FA667
TORVUS_ENERGY_CONTROLLER_INTERNAL_ID = 0x3B696A7A
TORVUS_GROVE_INTERNAL_ID = 0x72349FA8
TORVUS_LAGOON_INTERNAL_ID = 0x0678EB72
TORVUS_MAP_STATION_INTERNAL_ID = 0xC1EC09C2
TORVUS_PLAZA_INTERNAL_ID = 0x1127C9E6
TORVUS_TEMPLE_INTERNAL_ID = 0x9A2ACAFD
TRAINING_ACCESS_INTERNAL_ID = 0x92BE0B06
TRAINING_CHAMBER_INTERNAL_ID = 0x1495E65B
TRANSIT_TUNNEL_EAST_INTERNAL_ID = 0x3B9904FD
TRANSIT_TUNNEL_SOUTH_INTERNAL_ID = 0xAEECD662
TRANSIT_TUNNEL_WEST_INTERNAL_ID = 0xA9917664
TRANSPORT_TO_AGON_WASTES_INTERNAL_ID = 0xA1197E74
TRANSPORT_TO_SANCTUARY_FORTRESS_INTERNAL_ID = 0x1C4BDD72
TRANSPORT_TO_TEMPLE_GROUNDS_INTERNAL_ID = 0x8D385440
UNDERGROUND_TRANSPORT_INTERNAL_ID = 0x8C23B6C3
UNDERGROUND_TUNNEL_INTERNAL_ID = 0x70FC85B3
UNDERTEMPLE_INTERNAL_ID = 0x22690759
UNDERTEMPLE_ACCESS_INTERNAL_ID = 0x416E3BA9
UNDERTEMPLE_SHAFT_INTERNAL_ID = 0x4DC6C3E3
UNDERTRANSIT_ONE_INTERNAL_ID = 0x3236D73D
UNDERTRANSIT_TWO_INTERNAL_ID = 0x02A8C6E4
VENOMOUS_POND_INTERNAL_ID = 0xAF3C03B1

NAME_TO_ID_INTERNAL_ID = {
    "Abandoned Worksite": 0x8F9D66C0,
    "Ammo Station": 0xA0E73B00,
    "Brooding Ground": 0x3C156DDF,
    "Cache A": 0x97B812A2,
    "Cache B": 0xF04E9059,
    "Catacombs": 0xAD9D5D9F,
    "Catacombs Access": 0xC4F8E78E,
    "Controller Access": 0x94182918,
    "Crypt": 0x4280A031,
    "Crypt Tunnel": 0xA29D33C8,
    "Dark Arena Tunnel": 0xA1ECA232,
    "Dark Controller Access": 0x65B4E891,
    "Dark Falls": 0x9172B3EB,
    "Dark Forgotten Bridge": 0xFF42758B,
    "Dark Torvus Arena": 0xF5F50ABC,
    "Dark Torvus Energy Controller": 0x8E2AF917,
    "Dark Torvus Temple": 0x915E47CE,
    "Dark Torvus Temple Access": 0x6DB75FB4,
    "Dungeon": 0xFE8931FC,
    "Forgotten Bridge": 0x2406DD54,
    "Fortress Transport Access": 0x518D3511,
    "Gathering Access": 0x3AD40963,
    "Gathering Hall": 0xF9F219B0,
    "Gloom Vista": 0x279818B4,
    "Great Bridge": 0x069D4087,
    "Grove Access": 0x734255B0,
    "Hydrochamber Storage": 0x0112E539,
    "Hydrodynamo Shaft": 0x3CE4A4FB,
    "Hydrodynamo Station": 0x2A621228,
    "Main Energy Controller": 0xDE01532E,
    "Main Hydrochamber": 0x50039A19,
    "Meditation Vista": 0xA701265D,
    "Path of Roots": 0xC746378D,
    "Plaza Access": 0xE14A2729,
    "Poisoned Bog": 0xA7DA4FD1,
    "Polluted Mire": 0x8D13B482,
    "Portal Chamber (Dark)": 0x9F510909,
    "Portal Chamber (Light)": 0x554E4514,
    "Putrid Alcove": 0x0C8B7C06,
    "Ruined Alcove": 0x1D951459,
    "Sacrificial Chamber": 0xC1AE6ECF,
    "Sacrificial Chamber Tunnel": 0x7FF3F046,
    "Save Station 1": 0x59E57214,
    "Save Station 2": 0x0D580FBE,
    "Save Station A": 0x964465B0,
    "Save Station B": 0xA74BE553,
    "Temple Access": 0xE2F4F72A,
    "Temple Transport Access": 0xAA2FA667,
    "Torvus Energy Controller": 0x3B696A7A,
    "Torvus Grove": 0x72349FA8,
    "Torvus Lagoon": 0x0678EB72,
    "Torvus Map Station": 0xC1EC09C2,
    "Torvus Plaza": 0x1127C9E6,
    "Torvus Temple": 0x9A2ACAFD,
    "Training Access": 0x92BE0B06,
    "Training Chamber": 0x1495E65B,
    "Transit Tunnel East": 0x3B9904FD,
    "Transit Tunnel South": 0xAEECD662,
    "Transit Tunnel West": 0xA9917664,
    "Transport to Agon Wastes": 0xA1197E74,
    "Transport to Sanctuary Fortress": 0x1C4BDD72,
    "Transport to Temple Grounds": 0x8D385440,
    "Underground Transport": 0x8C23B6C3,
    "Underground Tunnel": 0x70FC85B3,
    "Undertemple": 0x22690759,
    "Undertemple Access": 0x416E3BA9,
    "Undertemple Shaft": 0x4DC6C3E3,
    "Undertransit One": 0x3236D73D,
    "Undertransit Two": 0x02A8C6E4,
    "Venomous Pond": 0xAF3C03B1,
}

DOCK_NAMES = {
    "Abandoned Worksite": {
        "North": 0,
        "South": 1,
    },
    "Ammo Station": {
        "North": 0,
    },
    "Brooding Ground": {
        "North": 0,
        "South": 1,
    },
    "Cache A": {
        "North": 0,
        "VirtualNorth1": 1,
    },
    "Cache B": {
        "East_04": 0,
        "East_07": 1,
    },
    "Catacombs": {
        "North": 0,
        "West": 1,
        "West (West)": 2,
        "VirtualSouth": 3,
        "VirtualNorth1": 4,
    },
    "Catacombs Access": {
        "East": 0,
        "West": 1,
    },
    "Controller Access": {
        "docks_docks_West": 0,
        "East1": 1,
    },
    "Crypt": {
        "East_A": 0,
        "East_B": 1,
        "VirtualNorth": 2,
        "North": 3,
    },
    "Crypt Tunnel": {
        "East": 0,
        "West": 1,
    },
    "Dark Arena Tunnel": {
        "North": 0,
        "South": 1,
    },
    "Dark Controller Access": {
        "docks_docks_West": 0,
        "East1": 1,
    },
    "Dark Falls": {
        "West": 0,
        "East": 1,
    },
    "Dark Forgotten Bridge": {
        "North": 0,
        "East": 1,
        "South": 2,
        "VirtualNorth": 3,
        "West": 4,
    },
    "Dark Torvus Arena": {
        "South": 0,
    },
    "Dark Torvus Energy Controller": {
        "group6_East": 0,
        "North1": 1,
    },
    "Dark Torvus Temple": {
        "NorthDoor": 0,
        "South": 1,
        "West": 2,
        "WestGenerator": 3,
        "EastTop": 4,
        "East": 5,
    },
    "Dark Torvus Temple Access": {
        "Easttop": 0,
        "Eastbottom": 1,
        "Westtop": 2,
        "West": 3,
    },
    "Dungeon": {
        "North": 0,
        "West (mid)": 1,
        "West (West)": 2,
        "VirtualSouth": 3,
        "VirtualNorth1": 4,
    },
    "Forgotten Bridge": {
        "North": 0,
        "East": 1,
        "South": 2,
        "VirtualNorth": 3,
        "West": 4,
    },
    "Fortress Transport Access": {
        "South": 0,
        "North": 1,
    },
    "Gathering Access": {
        "East": 0,
        "West": 1,
    },
    "Gathering Hall": {
        "East_0L": 0,
        "East_0P": 1,
        "VirtualNorth": 2,
        "North": 3,
    },
    "Gloom Vista": {
        "Docks_South": 0,
        "VirtualNorth": 1,
    },
    "Great Bridge": {
        "WestUpper": 0,
        "WestLower": 1,
        "North": 2,
        "EastC": 3,
        "EastB": 4,
        "Docks_SouthUpper1": 5,
    },
    "Grove Access": {
        "West": 0,
        "East": 1,
    },
    "Hydrochamber Storage": {
        "North": 0,
    },
    "Hydrodynamo Shaft": {
        "Docks_VirtualNorth": 0,
        "Docks_North": 1,
        "West": 2,
    },
    "Hydrodynamo Station": {
        "West_0J": 0,
        "West_0L": 1,
        "North_OK": 2,
        "East_0M": 3,
        "East_Save": 4,
        "VirtualNorth": 5,
        "East (floor to 0q)": 6,
    },
    "Main Energy Controller": {
        "SouthLight1": 0,
        "VirtualNorth": 1,
        "North1": 2,
    },
    "Main Hydrochamber": {
        "South": 0,
        "VirtualNorth": 1,
        "South2": 2,
    },
    "Meditation Vista": {
        "Docks_South": 0,
        "VirtualNorth": 1,
    },
    "Path of Roots": {
        "West": 0,
        "North": 1,
    },
    "Plaza Access": {
        "North": 0,
        "South": 1,
    },
    "Poisoned Bog": {
        "NorthUpper": 0,
        "West": 1,
        "SouthLower": 2,
        "South": 3,
        "East": 4,
    },
    "Polluted Mire": {
        "East": 0,
        "South": 1,
        "North": 2,
    },
    "Portal Chamber (Dark)": {
        "East": 0,
        "VirtualNorth": 1,
        "West": 2,
    },
    "Portal Chamber (Light)": {
        "East": 0,
        "VirtualNorth": 1,
        "West": 2,
    },
    "Putrid Alcove": {
        "West": 0,
        "South": 1,
    },
    "Ruined Alcove": {
        "West": 0,
        "South": 1,
    },
    "Sacrificial Chamber": {
        "South": 0,
        "East": 1,
        "West1": 2,
        "North": 3,
    },
    "Sacrificial Chamber Tunnel": {
        "North": 0,
        "South": 1,
    },
    "Save Station 1": {
        "Docks_Docks_North": 0,
    },
    "Save Station 2": {
        "Docks_Docks_Docks_West": 0,
    },
    "Save Station A": {
        "North": 0,
        "VirtualNorth1": 1,
    },
    "Save Station B": {
        "Docks_Docks_Docks_West": 0,
    },
    "Temple Access": {
        "Easttop": 0,
        "Eastbottom": 1,
        "Westtop": 2,
        "West": 3,
    },
    "Temple Transport Access": {
        "East": 0,
        "West": 1,
    },
    "Torvus Energy Controller": {
        "group6_East": 0,
        "North1": 1,
    },
    "Torvus Grove": {
        "East": 0,
        "South": 1,
        "North": 2,
    },
    "Torvus Lagoon": {
        "NorthUpper (0D_Hall)": 0,
        "West (0C_Hall)": 1,
        "SouthLower (0B_Hall)": 2,
        "South (00_Pickup)": 3,
        "East (0A_Hall)": 4,
    },
    "Torvus Map Station": {
        "Docks_Docks_North": 0,
    },
    "Torvus Plaza": {
        "South": 0,
    },
    "Torvus Temple": {
        "NorthDoor": 0,
        "South": 1,
        "West": 2,
        "WestGenerator": 3,
        "EastTop": 4,
        "East": 5,
    },
    "Training Access": {
        "North": 0,
        "South": 1,
    },
    "Training Chamber": {
        "South": 0,
        "East": 1,
        "West1": 2,
        "North": 3,
    },
    "Transit Tunnel East": {
        "West": 0,
        "South": 1,
    },
    "Transit Tunnel South": {
        "East": 0,
        "West": 1,
    },
    "Transit Tunnel West": {
        "South": 0,
        "East": 1,
    },
    "Transport to Agon Wastes": {
        "North": 0,
    },
    "Transport to Sanctuary Fortress": {
        "South": 0,
    },
    "Transport to Temple Grounds": {
        "West": 0,
    },
    "Underground Transport": {
        "East_04 (Top)": 0,
        "East_07 (Bottom)": 1,
    },
    "Underground Tunnel": {
        "South": 0,
        "North": 1,
    },
    "Undertemple": {
        "South": 0,
        "VirtualNorth": 1,
        "South2": 2,
    },
    "Undertemple Access": {
        "Docks_VirtualNorth": 0,
        "Docks_North": 1,
        "West": 2,
    },
    "Undertemple Shaft": {
        "West_0J": 0,
        "West_0L": 1,
        "North_OK": 2,
        "East_0M": 3,
        "Docks_Docks_East_Save": 4,
        "VirtualNorth": 5,
        "East_0M1 (Floor to 0q)": 6,
    },
    "Undertransit One": {
        "South": 0,
        "East": 1,
    },
    "Undertransit Two": {
        "West": 0,
        "South": 1,
    },
    "Venomous Pond": {
        "WestUpper": 0,
        "WestLower": 1,
        "North": 2,
        "EastC": 3,
        "EastB": 4,
        "Docks_SouthUpper1": 5,
    },
}
