from open_prime_rando.dol_patching.prime1.dol_patches import Prime1DolVersion

ALL_VERSIONS = [
    Prime1DolVersion(
        version="0-00",
        description="GC NTSC 0-00",
        build_string_address=0x803CC588,
        build_string=b"!#$MetroidBuildInfo!#$Build v1.088 10/29/2002 2:21:25",
        sda2_base=0x805B1D20,
        sda13_base=0x805AEBC0,
        cplayer_vtable=0x803D96E8,
        message_receiver_string_ref=0x803EFB90,
    ),
    Prime1DolVersion(
        version="0-01",
        description="GC NTSC 0-01",
        build_string_address=0x803CC768,
        build_string=b"!#$MetroidBuildInfo!#$Build v1.093 11/5/2002 19:50:01",
        sda2_base=0x805B1F00,
        sda13_base=0x805AEDA0,
        cplayer_vtable=0x803D98C8,
        message_receiver_string_ref=0x803EFD70,
    ),
    Prime1DolVersion(
        version="0-02",
        description="GC NTSC 0-02",
        build_string_address=0x803CD648,
        build_string=b"!#$MetroidBuildInfo!#$Build v1.111 3/10/2003 17:56:21",
        sda2_base=0x805B2DE0,
        sda13_base=0x805AFC40,
        cplayer_vtable=0x803DA7A8,
        message_receiver_string_ref=0x803F0BA8,
    ),
    Prime1DolVersion(
        version="pal",
        description="GC PAL",
        build_string_address=0x803B6924,
        build_string=b"!#$MetroidBuildInfo!#$Build v1.110 2/4/2003 22:16:07",
        sda2_base=0x80473E60,
        sda13_base=0x80470C60,
        cplayer_vtable=0x803C4B88,
        message_receiver_string_ref=0x803D7A28,
    ),
    Prime1DolVersion(
        version="kor",
        description="GC NTSC-K",
        build_string_address=0x803CC688,
        build_string=b"!#$MetroidBuildInfo!#$Build v1.097 12/19/2002 16:03:43",
        sda2_base=0x805B1A00,
        sda13_base=0x805AE8A0,
        cplayer_vtable=0x803D97E8,
        message_receiver_string_ref=0x803EFC90,
    ),
    Prime1DolVersion(
        version="jpn",
        description="GC NTSC-J",
        build_string_address=0x803B86CC,
        build_string=b"!#$MetroidBuildInfo!#$Build v1.111 2/11/2003 10:03:22",
        sda2_base=0x8059AE00,
        sda13_base=0x80597C20,
        cplayer_vtable=0x803C5B28,
        message_receiver_string_ref=0x803D89C8,
    ),
]
